==> wyvern-2.2.7.tar.bz2: wyvern-2.2.7/modules/mod_auth_db.c <==
/*****************************************************************************
 *
 * FILE:	mod_auth_db.c
 * DESCRIPTION:	Wyvern: authentication module with Berkeley DB
 * DATE:	Wed, Aug 28 2002
 * UPDATED:	Wed, Mar  1 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_auth_db.c,v 1.21 2006/03/01 13:21:37 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#include <errno.h>
#include <db.h>
#include "wyvern.h"
#include "auth.h"

/*****************************************************************************
 *
 *	definition of macros and structure
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_auth_db"

#define	DB_SUFFIX	".db"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void	wm_auth_db_init(void);
static void	wm_auth_db_finish(void);
static int	wm_auth_db_checker(Message *);
static char *	get_password(const char *, const char *);
static void	wm_auth_db_register_hooks(void);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static Cache *	authList = NULL;

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_auth_db_init
 * Description:		initialize cache for authentication table
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_auth_db_init(void)
{
  AuthProc	ap = { get_password, NULL, NULL };

  authList = wc_new_cache(WA_AUTH_TABLE_SIZE);
  if (authList == NULL) {
    wv_die("[%s] failed to initialize cache", WM_MODULE_ID);
  }
  wauth_set_subr(&ap);
}

/*
 * Function Name:	wm_auth_db_finish
 * Description:		release cache memory area
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_auth_db_finish(void)
{
  wauth_cleaner(authList);
}

/*****************************************************************************/

/*
 * Function Name:	wm_auth_db_checker
 * Description:		check user access
 * Arguments:		message
 * Return Value:	allow: 0, otherwise: -1
 */
static int
wm_auth_db_checker(msg)
	register Message *	msg;
{
  DBG(fputs("<auth_db_checker>\n", stderr);)
  return wauth_checker(authList, msg, DB_SUFFIX);
}

static char *
get_password(path, name)
	const char *	path;	/* path of password file */
	const char *	name;	/* username */
{
  register DB *		dp;
  char *		retval = NULL;

  if (path == NULL || name == NULL) {
    errno = EINVAL;
    return NULL;
  }

  dp = dbopen(path, O_RDONLY, 0600, DB_HASH, NULL);
  if (dp != NULL) {
    DBT	key;
    DBT	data;

    key.data = (char *)name;
    key.size = strlen(name);
    if (dp->get(dp, &key, &data, 0) == 0) {
      retval = ws_strndup(data.data, data.size);
    }
    dp->close(dp);
  }

#if	DEBUG
  if (retval != NULL) {
    wv_syslog(WLL_LOG_DEBUG, "[%s] found: %s=%s", WM_MODULE_ID, name, retval);
  }
  else {
    wv_syslog(WLL_LOG_DEBUG, "[%s] not found: %s", WM_MODULE_ID, name);
  }
#endif	/* DEBUG */

  return retval;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_auth_db_register_hooks(void)
{
  wm_hook_auth_checker(wm_auth_db_checker, HOOK_MIDDLE);
}

static wmModule	mod_auth_db = {
  WM_MAGIC_NUMBER_MAJOR,		/* version */
  WM_SECURE,				/* secure */
  WM_MODULE_ID,				/* name */
  "Authentication Module with Berkeley DB version 1.2.4",	/* info */
  "Kouichi ABE (WALL)",			/* author */
  "libauth",				/* depend */
  NULL,					/* handle */
  (void *)wm_auth_db_init,		/* init */
  (void *)wm_auth_db_finish,		/* finish */
  (void *)wm_auth_db_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_auth_db;
}

==> wyvern-2.2.10.tar.bz2: wyvern-2.2.10/modules/mod_auth_db.c <==
/*****************************************************************************
 *
 * FILE:	mod_auth_db.c
 * DESCRIPTION:	Wyvern: authentication module with Berkeley DB
 * DATE:	Wed, Aug 28 2002
 * UPDATED:	Sat, Aug 25 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_auth_db.c,v 1.22 2007/08/24 18:01:55 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#include <errno.h>
#include <db.h>
#include "wyvern.h"
#include "auth.h"

/*****************************************************************************
 *
 *	definition of macros and structure
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_auth_db"

#define	DB_SUFFIX	".db"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void	wm_auth_db_init(void);
static void	wm_auth_db_finish(void);
static int	wm_auth_db_checker(Message *);
static char *	get_password(const char *, const char *);
static void	wm_auth_db_register_hooks(void);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static Cache *	authList = NULL;

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_auth_db_init
 * Description:		initialize cache for authentication table
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_auth_db_init(void)
{
  AuthProc	ap = { get_password, NULL, NULL };

  authList = wc_new_cache(WA_AUTH_TABLE_SIZE);
  if (authList == NULL) {
    wv_die("[%s] failed to initialize cache", WM_MODULE_ID);
  }
  wauth_set_subr(&ap);
}

/*
 * Function Name:	wm_auth_db_finish
 * Description:		release cache memory area
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_auth_db_finish(void)
{
  wauth_cleaner(authList);
}

/*****************************************************************************/

/*
 * Function Name:	wm_auth_db_checker
 * Description:		check user access
 * Arguments:		message
 * Return Value:	allow: 0, otherwise: -1
 */
static int
wm_auth_db_checker(msg)
	register Message *	msg;
{
  DBG(fputs("<auth_db_checker>\n", stderr);)
  return wauth_checker(authList, msg, DB_SUFFIX);
}

static char *
get_password(path, name)
	const char *	path;	/* path of password file */
	const char *	name;	/* username */
{
  register DB *		dp;
  char *		status = NULL;

  if (path == NULL || name == NULL) {
    errno = EINVAL;
    return NULL;
  }

  dp = dbopen(path, O_RDONLY, 0600, DB_HASH, NULL);
  if (dp != NULL) {
    DBT	key;
    DBT	data;

    key.data = (char *)name;
    key.size = strlen(name);
    if (dp->get(dp, &key, &data, 0) == 0) {
      status = ws_strndup(data.data, data.size);
    }
    dp->close(dp);
  }

#if	DEBUG
  if (status != NULL) {
    wv_syslog(WLL_LOG_DEBUG, "[%s] found: %s=%s", WM_MODULE_ID, name, status);
  }
  else {
    wv_syslog(WLL_LOG_DEBUG, "[%s] not found: %s", WM_MODULE_ID, name);
  }
#endif	/* DEBUG */

  return status;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_auth_db_register_hooks(void)
{
  wm_hook_auth_checker(wm_auth_db_checker, HOOK_MIDDLE);
}

static wmModule	mod_auth_db = {
  WM_MAGIC_NUMBER_MAJOR,		/* version */
  WM_SECURE,				/* secure */
  WM_MODULE_ID,				/* name */
  "Authentication Module with Berkeley DB version 1.2.4",	/* info */
  "Kouichi ABE (WALL)",			/* author */
  "libauth",				/* depend */
  NULL,					/* handle */
  (void *)wm_auth_db_init,		/* init */
  (void *)wm_auth_db_finish,		/* finish */
  (void *)wm_auth_db_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_auth_db;
}
